@@ -1,5 +1,5 @@
 // Netify Agent
-// Copyright (C) 2015-2020 eGloo Incorporated <http://www.egloo.ca>
+// Copyright (C) 2015-2022 eGloo Incorporated <http://www.egloo.ca>
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -17,8 +17,6 @@
 #ifndef _ND_JSON_H
 #define _ND_JSON_H
 
-#define ND_JSON_TOKENER_DEPTH   16
-
 class ndJsonInitException : public runtime_error
 {
 public:
@@ -34,9 +32,20 @@ public:
 };
 
 void nd_json_to_string(const json &j, string &output, bool pretty = false);
+
 void nd_json_save_to_file(const json &j, const string &filename, bool pretty = false);
 void nd_json_save_to_file(const string &j, const string &filename);
 
+void nd_json_agent_hello(string &json_string);
+void nd_json_agent_status(json &j);
+
+void nd_json_protocols(string &json_string);
+
+void nd_json_add_interfaces(json &parent);
+void nd_json_add_devices(json &parent);
+void nd_json_add_stats(json &parent,
+    nd_packet_stats *stats, struct pcap_stat *pcap);
+
 typedef vector<string> ndJsonDataChunks;
 typedef map<string, ndJsonDataChunks> ndJsonData;
 
