@@ -1,5 +1,5 @@
 // Netify Agent
-// Copyright (C) 2015-2021 eGloo Incorporated <http://www.egloo.ca>
+// Copyright (C) 2015-2022 eGloo Incorporated <http://www.egloo.ca>
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -27,11 +27,14 @@
 #include <stdexcept>
 #include <string>
 #include <unordered_map>
+#include <unordered_set>
 #include <list>
 #include <vector>
+#include <set>
 #include <atomic>
 #include <regex>
 #include <algorithm>
+#include <mutex>
 
 #ifdef HAVE_ENDIAN_H
 #include <endian.h>
@@ -146,12 +149,19 @@ using namespace std;
 
 #include "netifyd.h"
 
+#include "nd-config.h"
 #include "nd-ndpi.h"
 #ifdef _ND_USE_NETLINK
 #include "nd-netlink.h"
 #endif
 #include "nd-json.h"
+#include "nd-apps.h"
+#include "nd-protos.h"
+#include "nd-risks.h"
+#include "nd-category.h"
 #include "nd-flow.h"
+#include "nd-flow-map.h"
+#include "nd-flow-parser.h"
 #include "nd-thread.h"
 #ifdef _ND_USE_CONNTRACK
 #include "nd-conntrack.h"
@@ -165,25 +175,31 @@ using namespace std;
 #include "nd-capture.h"
 
 // Enable to log discarded packets
-//#define _ND_LOG_PKT_DISCARD     1
+//#define _ND_LOG_PKT_DISCARD   1
 
 // Enable to log discarded flows
-//#define _ND_LOG_FLOW_DISCARD    1
+//#define _ND_LOG_FLOW_DISCARD  1
 
 // Enable DNS response debug logging
-//#define _ND_LOG_DNS_RESPONSE    1
+//#define _ND_LOG_DNS_RESPONSE  1
 
 // Enable DNS hint cache debug logging
-//#define _ND_LOG_DHC             1
+//#define _ND_LOG_DHC           1
 
 // Enable flow hash cache debug logging
-//#define _ND_LOG_FHC             1
+//#define _ND_LOG_FHC           1
 
 // Enable GTP tunnel dissection
-#define _ND_DISSECT_GTP       1
+#define _ND_DISSECT_GTP         1
 
-extern nd_global_config nd_config;
-extern atomic_uint nd_flow_count;
+extern nd_global_config *nd_config;
+extern ndFlowMap *nd_flow_buckets;
+
+atomic_uint nd_flow_count;
+
+nd_device nd_devices;
+nd_interface_addr_map nd_interface_addrs;
+nd_interface nd_interfaces;
 
 struct __attribute__((packed)) nd_mpls_header_t
 {
@@ -289,12 +305,12 @@ size_t ndPacketQueue::push(struct pcap_pkthdr *pkt_header, const uint8_t *pkt_da
     nd_dprintf("%s: packet queue push, new size: %lu\n",
         tag.c_str(), pkt_queue_size);
 #endif
-    if (pkt_queue_size >= nd_config.max_packet_queue) {
+    if (pkt_queue_size >= nd_config->max_packet_queue) {
 
         nd_dprintf("%s: packet queue full: %lu\n",
             tag.c_str(), pkt_queue_size);
 
-        size_t target = nd_config.max_packet_queue / ND_PKTQ_FLUSH_DIVISOR;
+        size_t target = nd_config->max_packet_queue / ND_PKTQ_FLUSH_DIVISOR;
 
         do {
             pop("flush");
@@ -340,20 +356,20 @@ void ndPacketQueue::pop(const string &oper)
 
 ndCaptureThread::ndCaptureThread(
     int16_t cpu,
-    nd_ifaces::iterator iface,
+    nd_interface::iterator iface,
     const uint8_t *dev_mac,
     ndSocketThread *thread_socket,
     const nd_detection_threads &threads_dpi,
-    nd_flow_map *flow_map, nd_packet_stats *stats,
+    nd_packet_stats *stats,
     ndDNSHintCache *dhc,
     uint8_t private_addr)
     : ndThread(iface->second, (long)cpu, true),
     iface(iface), thread_socket(thread_socket),
     capture_unknown_flows(ND_CAPTURE_UNKNOWN_FLOWS),
-    pcap(NULL), pcap_fd(-1), pcap_snaplen(ND_PCAP_SNAPLEN),
-    pcap_datalink_type(0), pkt_header(NULL), pkt_data(NULL),
-    ts_pkt_last(0),
-    flows(flow_map), stats(stats), dhc(dhc),
+    pcap(NULL), pcap_fd(-1), pcap_datalink_type(0),
+    pkt_header(NULL), pkt_data(NULL),
+    tv_epoch(0), ts_pkt_first(0), ts_pkt_last(0),
+    stats(stats), dhc(dhc),
     pkt_queue(iface->second),
     threads_dpi(threads_dpi), dpi_thread_id(rand() % threads_dpi.size())
 {
@@ -392,33 +408,15 @@ ndCaptureThread::~ndCaptureThread()
 
 void *ndCaptureThread::Entry(void)
 {
-    bool dump_flows = false;
-
+    int rc;
     struct ifreq ifr;
+    bool warnings = true;
 
-    do {
-        if (pcap == NULL && ! ShouldTerminate()) {
-            if (nd_ifreq(tag, SIOCGIFFLAGS, &ifr) == -1) {
-                sleep(1);
-                continue;
-            }
+    while (! ShouldTerminate() || ! pkt_queue.empty()) {
 
-            if (! (ifr.ifr_flags & IFF_UP)) {
-                nd_dprintf("%s: WARNING: interface is down.\n",
-                    tag.c_str());
-                sleep(1);
-                continue;
-            }
-
-            if ((pcap = OpenCapture()) == NULL) {
-                sleep(1);
-                continue;
-            }
-
-            pcap_datalink_type = pcap_datalink(pcap);
-
-            nd_dprintf("%s: capture started on CPU: %lu\n",
-                tag.c_str(), cpu >= 0 ? cpu : 0);
+        if (ShouldTerminate() && pcap != NULL) {
+            pcap_close(pcap);
+            pcap = NULL;
         }
 
         if (! pkt_queue.empty() && pthread_mutex_trylock(&lock) == 0) {
@@ -438,58 +436,33 @@ void *ndCaptureThread::Entry(void)
             pkt_queue.pop();
         }
 
-        if (! ShouldTerminate()) {
-            if (pcap_fd != -1) {
-                int rc, max_fd = 0;
-                struct timeval tv;
-                fd_set fds_read;
+        if (pcap != NULL) {
+            int max_fd = 0;
+            struct timeval tv;
+            fd_set fds_read;
 
-                FD_ZERO(&fds_read);
-                FD_SET(fd_ipc[0], &fds_read);
-                FD_SET(pcap_fd, &fds_read);
+            FD_ZERO(&fds_read);
+            FD_SET(pcap_fd, &fds_read);
 
-                memset(&tv, 0, sizeof(struct timeval));
+            memset(&tv, 0, sizeof(struct timeval));
 
-                if (pkt_queue.empty()) tv.tv_sec = 1;
-                tv.tv_usec = ND_TTL_PCAP_SELECT_USEC;
+            if (pkt_queue.empty()) tv.tv_sec = 1;
 
-                max_fd = max(fd_ipc[0], pcap_fd);
-                rc = select(max_fd + 1, &fds_read, NULL, NULL, &tv);
+            max_fd = max(fd_ipc[0], pcap_fd);
+            rc = select(max_fd + 1, &fds_read, NULL, NULL, &tv);
 
-                if (rc == -1)
-                    throw ndCaptureThreadException(strerror(errno));
+            if (rc == 0) continue;
+            else if (rc == -1)
+                throw ndCaptureThreadException(strerror(errno));
 
-                if (dump_flows && pthread_mutex_trylock(&lock) == 0) {
-                    if (ND_FLOW_DUMP_ESTABLISHED)
-                        DumpFlows();
-                    dump_flows = false;
-                    pthread_mutex_unlock(&lock);
-                }
+            if (! FD_ISSET(pcap_fd, &fds_read)) continue;
 
-                if (rc == 0) continue;
-
-                if (FD_ISSET(fd_ipc[0], &fds_read)) {
-                    uint32_t id = RecvIPC();
-
-                    if (id == (uint32_t)ND_SIG_CONNECT)
-                        dump_flows = true;
-                    else {
-                        nd_dprintf("%s: Unknown IPC ID: %u (ND_SIG_CONNECT: %u).\n",
-                            tag.c_str(), id, ND_SIG_CONNECT);
-                    }
-                }
-
-                if (! FD_ISSET(pcap_fd, &fds_read)) continue;
-            }
-
-            switch (pcap_next_ex(pcap, &pkt_header, &pkt_data)) {
-            case 0:
-                break;
-            case 1:
-                if (pthread_mutex_trylock(&lock) != 0) {
+            rc = 0;
+            while (ShouldTerminate() == false &&
+                (rc = pcap_next_ex(pcap, &pkt_header, &pkt_data)) > 0) {
 
+                if (pthread_mutex_trylock(&lock) != 0)
                     stats->pkt.queue_dropped += pkt_queue.push(pkt_header, pkt_data);
-                }
                 else {
                     bool from_queue = false;
 
@@ -509,59 +482,57 @@ void *ndCaptureThread::Entry(void)
 
                     if (from_queue) pkt_queue.pop();
                 }
-                break;
-            case -1:
-                nd_printf("%s: %s.\n", tag.c_str(), pcap_geterr(pcap));
+            }
+
+            if (rc < 0) {
+                if (rc == -1) {
+                    nd_printf("%s: %s.\n", tag.c_str(), pcap_geterr(pcap));
+                    if (pcap_file.size())
+                        Terminate();
+                    else
+                        sleep(1);
+                }
+                else if (rc == -2) {
+                    nd_dprintf(
+                        "%s: end of capture file: %s, flushing queued packets: %lu\n",
+                        tag.c_str(), pcap_file.c_str(), pkt_queue.size());
+
+                    Terminate();
+                }
+
                 pcap_close(pcap);
                 pcap = NULL;
-                pcap_fd = -1;
-                break;
-            case -2:
-                nd_dprintf(
-                    "%s: end of capture file: %s, flushing queued packets: %lu\n",
-                    tag.c_str(), pcap_file.c_str(), pkt_queue.size());
-                pcap_close(pcap);
-                pcap = NULL;
-                Terminate();
-                pcap_fd = -1;
-                break;
             }
         }
+        else if (! ShouldTerminate()) {
+            if (nd_ifreq(tag, SIOCGIFFLAGS, &ifr) == -1 ||
+                ! (ifr.ifr_flags & IFF_UP)) {
+                if (warnings) {
+                    nd_printf("%s: WARNING: interface not available.\n",
+                        tag.c_str());
+                    warnings = false;
+                }
+                sleep(1);
+                continue;
+            }
+
+            warnings = true;
+
+            if ((pcap = OpenCapture()) == NULL) {
+                sleep(1);
+                continue;
+            }
+
+            pcap_datalink_type = pcap_datalink(pcap);
+
+            nd_dprintf("%s: capture started on CPU: %lu\n",
+                tag.c_str(), cpu >= 0 ? cpu : 0);
+        }
     }
-    while (ShouldTerminate() == false || ! pkt_queue.empty());
 
     nd_dprintf(
         "%s: capture ended on CPU: %lu\n", tag.c_str(), cpu >= 0 ? cpu : 0);
-#if 0
-    nd_flow_map::iterator i = flows->begin();
 
-    while (i != flows->end()) {
-
-        if (thread_socket && (ND_FLOW_DUMP_UNKNOWN ||
-            i->second->detected_protocol.master_protocol > 0)) {
-
-            json j;
-
-            j["type"] = "flow_purge";
-            j["reason"] = "terminate";
-            j["interface"] = tag;
-            j["internal"] = iface->first;
-            j["established"] = false;
-
-            json jf;
-            i->second->json_encode(jf, ndFlow::ENCODE_STATS | ndFlow::ENCODE_TUNNELS);
-            j["flow"] = jf;
-
-            string json_string;
-            nd_json_to_string(j, json_string, false);
-            json_string.append("\n");
-
-            thread_socket->QueueWrite(json_string);
-        }
-
-        i++;
-    }
-#endif
     return NULL;
 }
 
@@ -573,15 +544,19 @@ pcap_t *ndCaptureThread::OpenCapture(void)
 
     if (pcap_file.size()) {
         if ((pcap_new = pcap_open_offline(pcap_file.c_str(), pcap_errbuf)) != NULL) {
+            tv_epoch = time(NULL);
             nd_dprintf("%s: reading from capture file: %s: v%d.%d\n",
                 tag.c_str(), pcap_file.c_str(),
                 pcap_major_version(pcap_new), pcap_minor_version(pcap_new));
         }
     }
     else {
-        pcap_new = pcap_open_live(tag.c_str(),
-            pcap_snaplen, 1, ND_PCAP_READ_TIMEOUT, pcap_errbuf
+        pcap_new = pcap_open_live(
+            tag.c_str(),
+            nd_config->max_capture_length,
+            1, ND_PCAP_READ_TIMEOUT, pcap_errbuf
         );
+
 #if 0
         if (pcap_new != NULL) {
             bool adapter = false;
@@ -610,12 +585,17 @@ pcap_t *ndCaptureThread::OpenCapture(void)
     if (pcap_new == NULL)
         nd_printf("%s: pcap_open: %s\n", tag.c_str(), pcap_errbuf);
     else {
+        if (pcap_file.empty()) {
+            if (pcap_setnonblock(pcap_new, 1, pcap_errbuf) == PCAP_ERROR)
+                nd_printf("%s: pcap_setnonblock: %s\n", tag.c_str(), pcap_errbuf);
+        }
+
         if ((pcap_fd = pcap_get_selectable_fd(pcap_new)) < 0)
             nd_dprintf("%s: pcap_get_selectable_fd: -1\n", tag.c_str());
 
-        nd_device_filter::const_iterator i = nd_config.device_filters.find(tag);
+        nd_device_filter::const_iterator i = nd_config->device_filters.find(tag);
 
-        if (i != nd_config.device_filters.end()) {
+        if (i != nd_config->device_filters.end()) {
 
             if (pcap_compile(pcap_new, &pcap_filter,
                 i->second.c_str(), 1, PCAP_NETMASK_UNKNOWN) < 0) {
@@ -637,45 +617,6 @@ pcap_t *ndCaptureThread::OpenCapture(void)
     return pcap_new;
 }
 
-// XXX: Not thread-safe!
-// XXX: Ensure the object is locked before calling.
-void ndCaptureThread::DumpFlows(void)
-{
-    unsigned flow_count = 0;
-
-    if (! thread_socket) return;
-
-    for (nd_flow_map::const_iterator i = flows->begin(); i != flows->end(); i++) {
-
-        if (i->second->flags.detection_complete.load() == false) continue;
-        if (! ND_FLOW_DUMP_UNKNOWN &&
-            i->second->detected_protocol.master_protocol == NDPI_PROTOCOL_UNKNOWN) continue;
-
-        json j;
-
-        j["type"] = "flow";
-        j["interface"] = tag;
-        j["interface"] = tag;
-        j["internal"] = iface->first;
-        j["established"] = true;
-
-        json jf;
-        i->second->json_encode(jf, ndFlow::ENCODE_METADATA);
-
-        j["flow"] = jf;
-
-        string json_string;
-        nd_json_to_string(j, json_string, false);
-        json_string.append("\n");
-
-        thread_socket->QueueWrite(json_string);
-
-        flow_count++;
-    }
-
-    nd_dprintf("%s: dumped %lu flow(s).\n", tag.c_str(), flow_count);
-}
-
 // XXX: Not thread-safe!
 // XXX: Ensure the object is locked before calling.
 int ndCaptureThread::GetCaptureStats(struct pcap_stat &stats)
@@ -689,7 +630,6 @@ int ndCaptureThread::GetCaptureStats(struct pcap_stat &stats)
 
 void ndCaptureThread::ProcessPacket(void)
 {
-    nd_flow_insert fi;
     ndFlow *nf, flow(iface);
 
     const struct ether_header *hdr_eth = NULL;
@@ -715,17 +655,26 @@ void ndCaptureThread::ProcessPacket(void)
             pkt_header->ts.tv_usec /
             (1000000 / ND_DETECTION_TICKS);
 
-    if (ts_pkt_last > ts_pkt) ts_pkt = ts_pkt_last;
+    if (pcap_file.size()) {
+        if (ts_pkt_first == 0)
+            ts_pkt_first = ts_pkt;
 
-    if (ND_REPLAY_DELAY && ts_pkt_last && pcap_file.size()) {
-        useconds_t delay = useconds_t(ts_pkt - ts_pkt_last) * 1000;
-        //nd_dprintf("%s: pkt delay: %lu\n", tag.c_str(), delay);
-        if (delay) {
-            pthread_mutex_unlock(&lock);
-            usleep(delay);
-            pthread_mutex_lock(&lock);
+        ts_pkt = (ts_pkt - ts_pkt_first) + (tv_epoch * 1000);
+
+        if (ts_pkt_last > ts_pkt) ts_pkt = ts_pkt_last;
+
+        if (ND_REPLAY_DELAY && ts_pkt_last) {
+            useconds_t delay = useconds_t(ts_pkt - ts_pkt_last) * 1000;
+            //nd_dprintf("%s: pkt delay: %lu\n", tag.c_str(), delay);
+            if (delay) {
+                pthread_mutex_unlock(&lock);
+                usleep(delay);
+                pthread_mutex_lock(&lock);
+            }
         }
     }
+    else
+        if (ts_pkt_last > ts_pkt) ts_pkt = ts_pkt_last;
 
     ts_pkt_last = ts_pkt;
 
@@ -735,6 +684,15 @@ void ndCaptureThread::ProcessPacket(void)
 
     switch (pcap_datalink_type) {
     case DLT_NULL:
+        if (pkt_header->caplen < sizeof(uint32_t)) {
+            stats->pkt.discard++;
+            stats->pkt.discard_bytes += pkt_header->len;
+#ifdef _ND_LOG_PKT_DISCARD
+            nd_dprintf("%s: discard: Truncated packet.", tag.c_str());
+#endif
+            return;
+        }
+
         switch (ntohl(*((uint32_t *)pkt_data))) {
         case 2:
             type = ETHERTYPE_IP;
@@ -758,6 +716,15 @@ void ndCaptureThread::ProcessPacket(void)
         break;
 
     case DLT_EN10MB:
+        if (pkt_header->caplen < sizeof(struct ether_header)) {
+            stats->pkt.discard++;
+            stats->pkt.discard_bytes += pkt_header->len;
+#ifdef _ND_LOG_PKT_DISCARD
+            nd_dprintf("%s: discard: Truncated packet.", tag.c_str());
+#endif
+            return;
+        }
+
         hdr_eth = reinterpret_cast<const struct ether_header *>(pkt_data);
         type = ntohs(hdr_eth->ether_type);
         l2_len = sizeof(struct ether_header);
@@ -779,6 +746,15 @@ void ndCaptureThread::ProcessPacket(void)
         break;
 
     case DLT_LINUX_SLL:
+        if (pkt_header->caplen < sizeof(struct sll_header)) {
+            stats->pkt.discard++;
+            stats->pkt.discard_bytes += pkt_header->len;
+#ifdef _ND_LOG_PKT_DISCARD
+            nd_dprintf("%s: discard: Truncated packet.", tag.c_str());
+#endif
+            return;
+        }
+
         hdr_sll = reinterpret_cast<const struct sll_header *>(pkt_data);
         type = hdr_sll->sll_protocol;
         l2_len = SLL_HDR_LEN;
@@ -801,11 +777,29 @@ void ndCaptureThread::ProcessPacket(void)
         return;
     }
 
+    if (l2_len > pkt_header->caplen) {
+        stats->pkt.discard++;
+        stats->pkt.discard_bytes += pkt_header->len;
+#ifdef _ND_LOG_PKT_DISCARD
+        nd_dprintf("%s: discard: layer-2 length is beyond capture length.\n",
+            tag.c_str());
+#endif
+    }
+
     while (true) {
         if (type == ETHERTYPE_VLAN) {
-            vlan_packet = 1;
+            if (l2_len + 4 > pkt_header->caplen) {
+                stats->pkt.discard++;
+                stats->pkt.discard_bytes += pkt_header->len;
+#ifdef _ND_LOG_PKT_DISCARD
+                nd_dprintf("%s: discard: layer-2 + VLAN length is beyond capture length.\n",
+                    tag.c_str());
+#endif
+            }
+
             // TODO: Replace with struct vlan_tag from <pcap/vlan.h>
             // See: https://en.wikipedia.org/wiki/IEEE_802.1Q
+            vlan_packet = 1;
             flow.vlan_id = ((pkt_data[l2_len] << 8) + pkt_data[l2_len + 1]) & 0xFFF;
             type = (pkt_data[l2_len + 2] << 8) + pkt_data[l2_len + 3];
             l2_len += VLAN_TAG_LEN;
@@ -820,14 +814,42 @@ void ndCaptureThread::ProcessPacket(void)
             type = ETHERTYPE_IP;
             l2_len += 4;
 
+            if (l2_len > pkt_header->caplen) {
+                stats->pkt.discard++;
+                stats->pkt.discard_bytes += pkt_header->len;
+#ifdef _ND_LOG_PKT_DISCARD
+                nd_dprintf("%s: discard: layer-2 + MPLS length is beyond capture length.\n",
+                    tag.c_str());
+#endif
+            }
+
             while (! mpls.mpls.s) {
                 l2_len += 4;
+
+                if (l2_len > pkt_header->caplen) {
+                    stats->pkt.discard++;
+                    stats->pkt.discard_bytes += pkt_header->len;
+#ifdef _ND_LOG_PKT_DISCARD
+                    nd_dprintf("%s: discard: layer-2 + MPLS length is beyond capture length.\n",
+                        tag.c_str());
+#endif
+                }
                 mpls.u32 = ntohl(*((uint32_t *)&pkt_data[l2_len]));
             }
         }
         else if (type == ETHERTYPE_PPPOE) {
             stats->pkt.pppoe++;
             type = ETHERTYPE_IP;
+
+            if (l2_len + 6 > pkt_header->caplen) {
+                stats->pkt.discard++;
+                stats->pkt.discard_bytes += pkt_header->len;
+#ifdef _ND_LOG_PKT_DISCARD
+                nd_dprintf("%s: discard: layer-2 + PPP length is beyond capture length.\n",
+                    tag.c_str());
+#endif
+            }
+
             ppp_proto = (uint16_t)(
                 _ND_PPP_PROTOCOL(pkt_data + l2_len + 6)
             );
@@ -842,6 +864,15 @@ void ndCaptureThread::ProcessPacket(void)
             }
 
             l2_len += 8;
+
+            if (l2_len > pkt_header->caplen) {
+                stats->pkt.discard++;
+                stats->pkt.discard_bytes += pkt_header->len;
+#ifdef _ND_LOG_PKT_DISCARD
+                nd_dprintf("%s: discard: layer-2 + PPP length is beyond capture length.\n",
+                    tag.c_str());
+#endif
+            }
         }
         else if (type == ETHERTYPE_PPPOEDISC) {
             stats->pkt.pppoe++;
@@ -859,6 +890,15 @@ void ndCaptureThread::ProcessPacket(void)
     stats->pkt.vlan += vlan_packet;
 
 nd_process_ip:
+    if (l2_len + sizeof(struct ip) > pkt_header->caplen) {
+        stats->pkt.discard++;
+        stats->pkt.discard_bytes += pkt_header->len;
+#ifdef _ND_LOG_PKT_DISCARD
+        nd_dprintf("%s: discard: layer-2 + IP length is beyond capture length.\n",
+            tag.c_str());
+#endif
+    }
+
     hdr_ip = reinterpret_cast<const struct ip *>(&pkt_data[l2_len]);
 
     flow.ip_version = hdr_ip->ip_v;
@@ -918,19 +958,7 @@ nd_process_ip:
 #endif
             return;
         }
-#if 0
-        // XXX: Disabled, drops all large packets.  Investigate.
 
-        if ((pkt_header->caplen - l2_len) < ntohs(hdr_ip->ip_len)) {
-            stats->pkt.discard++;
-            stats->pkt.discard_bytes += pkt_header->len;
-#ifdef _ND_LOG_PKT_DISCARD
-            nd_dprintf("%s: discard: (pkt_header->caplen[%hu] - l2_len[%hu](%hu)) < hdr_ip->ip_len[%hu]\n",
-                tag.c_str(), pkt_header->caplen, l2_len, pkt_header->caplen - l2_len, ntohs(hdr_ip->ip_len));
-#endif
-            return;
-        }
-#endif
         addr_cmp = memcmp(&hdr_ip->ip_src, &hdr_ip->ip_dst, 4);
 
         if (addr_cmp < 0) {
@@ -964,7 +992,7 @@ nd_process_ip:
         l4_len = ntohs(hdr_ip6->ip6_ctlun.ip6_un1.ip6_un1_plen);
         flow.ip_protocol = hdr_ip6->ip6_ctlun.ip6_un1.ip6_un1_nxt;
 
-        if (ndpi_handle_ipv6_extension_headers(NULL, &l4, &l4_len, &flow.ip_protocol)) {
+        if (ndpi_handle_ipv6_extension_headers(l3_len, &l4, &l4_len, &flow.ip_protocol)) {
             stats->pkt.discard++;
             stats->pkt.discard_bytes += pkt_header->len;
 #ifdef _ND_LOG_PKT_DISCARD
@@ -1014,6 +1042,17 @@ nd_process_ip:
 #endif
         return;
     }
+
+    if (l2_len + l3_len + l4_len > pkt_header->caplen) {
+        stats->pkt.discard++;
+        stats->pkt.discard_bytes += pkt_header->len;
+#ifdef _ND_LOG_PKT_DISCARD
+        nd_dprintf("%s: discard: L2 + L3 + L4 length is beyond capture length.\n",
+            tag.c_str());
+#endif
+        return;
+    }
+
 #if _ND_DISSECT_GTP
     if (l4_len > 8 && flow.ip_protocol == IPPROTO_UDP) {
         hdr_udp = reinterpret_cast<const struct udphdr *>(l4);
@@ -1118,9 +1157,31 @@ nd_process_ip:
 #endif
     switch (flow.ip_protocol) {
     case IPPROTO_TCP:
-        if (l4_len >= 20) {
+        stats->pkt.tcp++;
+
+        if (l4_len < 20) {
+            stats->pkt.discard++;
+            stats->pkt.discard_bytes += pkt_header->len;
+#ifdef _ND_LOG_PKT_DISCARD
+            nd_dprintf("%s: discard: TCP header too small.\n",
+                tag.c_str());
+#endif
+            return;
+        }
+        else {
             hdr_tcp = reinterpret_cast<const struct tcphdr *>(l4);
-            stats->pkt.tcp++;
+
+            uint16_t hdr_size = hdr_tcp->th_off * 4;
+
+            if (hdr_size < 20 || hdr_size > l4_len) {
+                stats->pkt.discard++;
+                stats->pkt.discard_bytes += pkt_header->len;
+    #ifdef _ND_LOG_PKT_DISCARD
+                nd_dprintf("%s: discard: unexpected TCP payload length.\n",
+                    tag.c_str());
+    #endif
+                return;
+            }
 
             if (addr_cmp < 0) {
                 flow.lower_port = hdr_tcp->th_sport;
@@ -1142,37 +1203,58 @@ nd_process_ip:
             }
 
             pkt = reinterpret_cast<const uint8_t *>(l4 + (hdr_tcp->th_off * 4));
-            pkt_len = l4_len - (hdr_tcp->th_off * 4);
+            pkt_len = l4_len - hdr_size;
         }
+
         break;
 
     case IPPROTO_UDP:
-        if (l4_len >= 8) {
-            hdr_udp = reinterpret_cast<const struct udphdr *>(l4);
-            stats->pkt.udp++;
+        stats->pkt.udp++;
 
-            if (addr_cmp < 0) {
+        if (l4_len < 8) {
+            stats->pkt.discard++;
+            stats->pkt.discard_bytes += pkt_header->len;
+#ifdef _ND_LOG_PKT_DISCARD
+            nd_dprintf("%s: discard: UDP header too small.\n",
+                tag.c_str());
+#endif
+            return;
+        }
+
+        hdr_udp = reinterpret_cast<const struct udphdr *>(l4);
+
+        if (addr_cmp < 0) {
+            flow.lower_port = hdr_udp->uh_sport;
+            flow.upper_port = hdr_udp->uh_dport;
+        }
+        else if (addr_cmp > 0) {
+            flow.lower_port = hdr_udp->uh_dport;
+            flow.upper_port = hdr_udp->uh_sport;
+        }
+        else {
+            if (hdr_udp->uh_sport < hdr_udp->uh_dport) {
                 flow.lower_port = hdr_udp->uh_sport;
                 flow.upper_port = hdr_udp->uh_dport;
             }
-            else if (addr_cmp > 0) {
+            else {
                 flow.lower_port = hdr_udp->uh_dport;
                 flow.upper_port = hdr_udp->uh_sport;
             }
-            else {
-                if (hdr_udp->uh_sport < hdr_udp->uh_dport) {
-                    flow.lower_port = hdr_udp->uh_sport;
-                    flow.upper_port = hdr_udp->uh_dport;
-                }
-                else {
-                    flow.lower_port = hdr_udp->uh_dport;
-                    flow.upper_port = hdr_udp->uh_sport;
-                }
-            }
-
-            pkt = reinterpret_cast<const uint8_t *>(l4 + sizeof(struct udphdr));
-            pkt_len = ntohs(hdr_udp->uh_ulen) - sizeof(struct udphdr);
         }
+
+        if (ntohs(hdr_udp->uh_ulen) != l4_len) {
+            stats->pkt.discard++;
+            stats->pkt.discard_bytes += pkt_header->len;
+#ifdef _ND_LOG_PKT_DISCARD
+            nd_dprintf("%s: discard: unexpected UDP data length.\n",
+                tag.c_str());
+#endif
+            return;
+        }
+
+        pkt = reinterpret_cast<const uint8_t *>(l4 + sizeof(struct udphdr));
+        pkt_len = ntohs(hdr_udp->uh_ulen) - sizeof(struct udphdr);
+
         break;
 
     case IPPROTO_ICMP:
@@ -1192,11 +1274,11 @@ nd_process_ip:
 
     flow.hash(tag);
     flow_digest.assign((const char *)flow.digest_lower, SHA1_DIGEST_LENGTH);
-    fi.first = flows->find(flow_digest);
 
-    if (fi.first != flows->end()) {
+    nf = nd_flow_buckets->Lookup(flow_digest);
+
+    if (nf != NULL) {
         // Flow exists in map.
-        nf = fi.first->second;
 
         if (addr_cmp != nf->direction) {
 #if _ND_DISSECT_GTP
@@ -1225,13 +1307,13 @@ nd_process_ip:
         }
     }
     else {
-        if (nd_config.max_flows > 0 && nd_flow_count + 1 > nd_config.max_flows) {
+        if (nd_config->max_flows > 0 && nd_flow_count + 1 > nd_config->max_flows) {
             stats->pkt.discard++;
             stats->pkt.discard_bytes += pkt_header->len;
             stats->flow.dropped++;
 #ifdef _ND_LOG_FLOW_DISCARD
             nd_dprintf("%s: discard: maximum flows exceeded: %u\n",
-                tag.c_str(), nd_config.max_flows);
+                tag.c_str(), nd_config->max_flows);
 #endif
             return;
         }
@@ -1241,9 +1323,7 @@ nd_process_ip:
 
         nf->direction = addr_cmp;
 
-        fi = flows->insert(nd_flow_pair(flow_digest, nf));
-
-        if (! fi.second) {
+        if (nd_flow_buckets->Insert(flow_digest, nf) != NULL) {
             // Flow exists in map!  Impossible!
             throw ndCaptureThreadException(strerror(EINVAL));
         }
@@ -1260,6 +1340,20 @@ nd_process_ip:
         else
             nf->origin = ndFlow::ORIGIN_UPPER;
 
+        // Refine according to the lowest port address
+        // XXX: The lowest port is likely the destination.
+        switch (nf->ip_protocol) {
+        case IPPROTO_TCP:
+        case IPPROTO_SCTP:
+        case IPPROTO_UDP:
+        case IPPROTO_UDPLITE:
+            if (ntohs(nf->lower_port) < ntohs(nf->upper_port))
+                nf->origin = ndFlow::ORIGIN_UPPER;
+            else if (ntohs(nf->lower_port) > ntohs(nf->upper_port))
+                nf->origin = ndFlow::ORIGIN_LOWER;
+            break;
+        }
+
         // Try to refine flow origin for TCP flows using SYN/ACK flags
         if (nf->ip_protocol == IPPROTO_TCP) {
 
@@ -1382,27 +1476,23 @@ nd_process_ip:
             }
 #else
             if (is_query && nf->flags.detection_complete.load()) {
-                nf->flags.dhc_hit = 0;
-                nf->flags.detection_complete = 0;
-                // XXX: Moot...
-                //if (fi.first != flows->end()) {
-                    nf->lower_bytes = flow.lower_bytes;
-                    nf->upper_bytes = flow.upper_bytes;
-                    nf->lower_packets = flow.lower_packets;
-                    nf->upper_packets = flow.upper_packets;
-                    nf->total_packets = flow.total_packets;
-                    nf->detection_packets = 0;
-                //}
+                nf->flags.dhc_hit = false;
+                nf->flags.detection_complete = false;
+                nf->lower_bytes = flow.lower_bytes;
+                nf->upper_bytes = flow.upper_bytes;
+                nf->lower_packets = flow.lower_packets;
+                nf->upper_packets = flow.upper_packets;
+                nf->total_packets = flow.total_packets;
+                nf->detection_packets = 0;
             }
 #endif
         }
     }
 
-    if (! nf->flags.detection_complete.load()
-        || (nf->ip_protocol == IPPROTO_UDP &&
-            nf->total_packets <= nd_config.max_udp_pkts)
-        || (nf->ip_protocol == IPPROTO_TCP &&
-            nf->total_packets <= nd_config.max_tcp_pkts)) {
+    if (nf->flags.detection_complete.load() == false &&
+        nf->flags.detection_expired.load() == false &&
+        nf->detection_packets <= nd_config->max_detection_pkts &&
+        nf->queued.load() <= nd_config->max_detection_pkts) {
 
         if (nf->dpi_thread_id < 0) {
             nf->dpi_thread_id = dpi_thread_id;
