@@ -1,5 +1,5 @@
 // Netify Agent
-// Copyright (C) 2015-2019 eGloo Incorporated <http://www.egloo.ca>
+// Copyright (C) 2015-2022 eGloo Incorporated <http://www.egloo.ca>
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -112,7 +112,7 @@ void ndInotify::RefreshWatches(void)
             IN_DELETE_SELF | IN_CLOSE_WRITE | IN_MODIFY);
 
         if (i->second->wd < 0) {
-            nd_debug_printf("Error creating inotify watch: %s: %s\n",
+            nd_dprintf("Error creating inotify watch: %s: %s\n",
                 i->first.c_str(), strerror(errno));
         }
         else
@@ -145,7 +145,7 @@ void ndInotify::ProcessEvent(void)
                         ((iev->mask & IN_DELETE_SELF) ||
                         (iev->mask & IN_MODIFY) || (iev->mask & IN_CLOSE_WRITE))) {
 
-                        nd_debug_printf("File event occured: %s [%s]\n",
+                        nd_dprintf("File event occured: %s [%s]\n",
                             watch->first.c_str(),
                             (iev->mask & IN_DELETE_SELF) ? "DELETE_SELF" :
                             (iev->mask & IN_MODIFY) ? "MODIFY" :
