@@ -1,5 +1,5 @@
 // Netify Agent
-// Copyright (C) 2015-2021 eGloo Incorporated <http://www.egloo.ca>
+// Copyright (C) 2015-2022 eGloo Incorporated <http://www.egloo.ca>
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -25,13 +25,16 @@
 #include <map>
 #include <queue>
 #include <list>
+#include <set>
 #include <sstream>
 #include <stdexcept>
 #include <string>
 #include <unordered_map>
+#include <unordered_set>
 #include <vector>
 #include <atomic>
 #include <regex>
+#include <mutex>
 
 #include <arpa/inet.h>
 #include <errno.h>
@@ -79,11 +82,16 @@ using namespace std;
 
 #include "netifyd.h"
 
+#include "nd-config.h"
 #include "nd-ndpi.h"
 #ifdef _ND_USE_NETLINK
 #include "nd-netlink.h"
 #endif
 #include "nd-json.h"
+#include "nd-apps.h"
+#include "nd-protos.h"
+#include "nd-risks.h"
+#include "nd-category.h"
 #include "nd-flow.h"
 #include "nd-thread.h"
 #ifdef _ND_USE_CONNTRACK
@@ -95,7 +103,7 @@ using namespace std;
 
 #define _ND_SOCKET_PROC_NET_UNIX    "/proc/net/unix"
 
-extern nd_global_config nd_config;
+extern nd_global_config *nd_config;
 
 ndSocketLocal::ndSocketLocal(ndSocket *base, const string &node)
     : base(base), valid(false)
@@ -679,16 +687,16 @@ ndSocketThread::ndSocketThread(int16_t cpu)
     : ndThread("nd-socket", (long)cpu)
 {
     vector<pair<string, string> >::const_iterator i;
-    for (i = nd_config.socket_host.begin();
-        i != nd_config.socket_host.end(); i++) {
+    for (i = nd_config->socket_host.begin();
+        i != nd_config->socket_host.end(); i++) {
         ndSocketServerRemote *skt;
         skt = new ndSocketServerRemote((*i).first, (*i).second);
         skt->SetBlockingMode(false);
         servers[skt->GetDescriptor()] = skt;
     }
     vector<string>::const_iterator j;
-    for (j = nd_config.socket_path.begin();
-        j != nd_config.socket_path.end(); j++) {
+    for (j = nd_config->socket_path.begin();
+        j != nd_config->socket_path.end(); j++) {
         ndSocketServerLocal *skt;
         skt = new ndSocketServerLocal((*j));
         skt->SetBlockingMode(false);
